option explicit

' Movie Battles II diagnostic script
' Version: 1.1
' Author: Puppytine, https://community.moviebattles.org/members/puppytine.200/

' true constants
Const _
  SSFMOpenForRead = 0, _
  ErrOpenFile = 0, _
  ErrReadFile = 1, _
  ErrEnumDir = 2, _
  ErrEnumFile = 3

' settings
' IgnoreMBIIDir  already handled by the MBII Launcher
Const _
  BufSize = 500000, _
  IgnoreMBIIDir = true, _ 
  sLogNameFiles = "MBII diag files.txt", _
  sLogNameMods = "MBII diag mods.txt", _
  sProcesses = "jamp.exe jasp.exe mbii.x86.exe"

' localizable
const _
  sErrorNotInGameData = "This script must be placed into ""Jedi Academy\GameData"" folder. Please move it to that folder.", _
  sErrorEnumDir = "ERROR: Can't get directory in ""%s"": ""%s""", _
  sErrorEnumFile = "ERROR: Can't get file in ""%s"": ""%s""", _
  sErrorPrivilege = "WARNING: Cannot get debug privilege: ", _
  sErrorVB = "Nbr: %s Src: %s Desc: %s", _
  sFinished = "All tasks are completed.\nLog file created: ""%s""\nDo you want to open it?", _
  sMD5Fmt = "%s %s %s bytes", _
  sModuleFmt = "%s: %s\n%s bytes, %s %s\n%s\n", _
  sScriptName = "MBII Diagnostic Script", _
  sStartTime = "Log file created on: "

dim OutFile, fso, OutFilePath, IsModules, md5, ErrStr, HasErr, ErrDbgPriv

function RepNewLine(ByVal s)
RepNewLine = Replace(s, "\n", vbNewLine)
end function

function DoReplace(ByVal s, ByVal t)
if IsNull(t) then t = "<null>"
DoReplace = Replace(s, "%s", t, 1, 1)
end function

function Format(ByVal s, ByVal args)
s = RepNewLine(s)
dim t
if IsArray(args) then
 for each t in args
  s = DoReplace(s, t)
 next
 else s = DoReplace(s, args)
end if
Format = s
end function 

function GetErr
' this proc must be very short and simple, since there is only one err object, so no second error should be allowed
HasErr = err.number <> 0
if HasErr then ErrStr = Format(sErrorVB, array(Err.Number, Err.Source, Err.Description))
end function

function CheckError(ByRef res)
if err.Number <> 0 then res = Format(sErrorVB, array(Err.Number, Err.Source, Err.Description))
CheckError = err.Number <> 0
end function

function BytesToHex(aBytes)
dim h, x
for x = 1 to lenb(aBytes)
 h = hex(ascb(midb((aBytes), x, 1)))
 if len(h) = 1 then h = "0" & h
 BytesToHex = BytesToHex & h
next
end function

function SizeOfFile(sPath)
dim f
set f = fso.GetFile(sPath)
SizeOfFile = f.Size
end function

Function MD5OfFile(sPath)
dim fs, x
fs = SizeOfFile(sPath)
if fs = 0 then 
 MD5OfFile = "<empty file>"
 exit Function
end if
REM if fs > 5000000 then 
 REM MD5OfFile = "file is too big (debug stuff)"
 REM exit Function
REM end if
set x = WScript.CreateObject("SAPI.spFileStream")
on error resume next
x.Open sPath, SSFMOpenForRead
if CheckError(MD5OfFile) then exit function
on error goto 0
Dim j, r, b
if IsEmpty(md5) then set md5 = CreateObject("System.Security.Cryptography.MD5CryptoServiceProvider")
md5.Initialize()
do while true
 if fs < BufSize then j = fs else j = BufSize 
 on error resume next
 x.Read b, j
 if CheckError(MD5OfFile) then exit function
 on error goto 0
 if fs < BufSize then 
  md5.TransformFinalBlock (b), 0, j
  exit do
  else md5.TransformBlock (b), 0, j, (b) , 0  
 end if
 fs = fs - j
loop
MD5OfFile = BytesToHex(md5.Hash)
End Function

sub CreateOutFile
if not IsEmpty(OutFile) then Exit sub
dim s, t
s = CreateObject("Wscript.Shell").SpecialFolders("Mydocuments")
if Right(s, 1) <> "\" THEN s = s & "\"
if IsModules then t = sLogNameMods else t = sLogNameFiles
s = s & t
Set OutFile = fso.CreateTextFile(s, True)
OutFilePath = s
WriteLn(sStartTime & now)
if not IsEmpty(ErrDbgPriv) then WriteLn(sErrorPrivilege & ErrDbgPriv)
end sub

sub WriteLn(ByVal Line)
CreateOutFile
OutFile.WriteLine(Line)
end sub

sub WriteLnF(ByVal Line, ByVal args)
WriteLn(Format(Line, args))
end sub

sub HashDir(Path)
if IgnoreMBIIDir then if LCase(fso.GetFileName(Path)) = "mbii" then exit sub
Dim f, f1, fc, s, b
Set f = fso.GetFolder(Path)
Set fc = f.SubFolders

on error resume next
For Each f1 in fc
 b = CheckError(s) 
 on error goto 0
 if b then WriteLnF vbNewLine + sErrorEnumDir, array(Path, s) else HashDir(f1.Path)
 on error resume next
next
on error goto 0

Set fc = f.Files
dim c

on error resume next
c = fc.Count
b = CheckError(s)
on error goto 0

if b then 
 WriteLnF sErrorEnumFile, array(Path, s)
 exit sub
end if
if c = 0 then exit sub
WriteLn(vbNewLine & Path & ":")
on error resume next
For Each f1 in fc
 b = CheckError(s) 
 on error goto 0
 if b then WriteLnF sErrorEnumFile, array(path, s) else WriteLnF sMD5Fmt, array(MD5OfFile(f1.Path), f1.Name, f1.Size)
 on error resume next
next
end sub

function MainFiles
Set fso = CreateObject("Scripting.FileSystemObject")
Dim d
d = fso.GetParentFolderName(WScript.ScriptFullName)
if LCase(fso.GetFileName(d)) <> "gamedata" then 
 MsgBox sErrorNotInGameData, vbOKOnly or vbCritical, sScriptName
 MainFiles = false
 exit function
end if
WriteLn("Directory: " & d & vbNewLine)
HashDir(d)
MainFiles = true
end function

function MainModules
IsModules = true
MainModules = false
dim p, w, o, r, m, q, s, i

on error resume next
set w = GetObject("winmgmts:\\.")
if err.number <> 0 then 
 on error goto 0
 ' we really, really need this service...
 WScript.CreateObject("WScript.Shell").Run "sc config winmgmt start= demand", 0, true
 set w = GetObject("winmgmts:\\.")
end if
on error goto 0

on error resume next
w.Security_.Privileges.AddAsString "SeDebugPrivilege", True
GetErr
on error goto 0
' can't write to log now, we don't know yet which file we should use
if HasErr then ErrDbgPriv = ErrStr

q = "SELECT handle FROM Win32_Process WHERE name = '" & Replace(sProcesses, " ", "' OR name = '") & "'"
set p = w.ExecQuery(q)
if p.Count = 0 then exit function
Set fso = CreateObject("Scripting.FileSystemObject")
for each o in p
 q = "ASSOCIATORS OF {Win32_Process.Handle=""" & o.handle & """} WHERE ResultClass = CIM_DataFile"
 WriteLn(o.handle & ":" & vbNewLine & "============================" & vbNewLine)
 set r = w.ExecQuery(q)
 i = 1
 for each m in r
  s = m.Name
  WriteLnF sModuleFmt, array(i, s, m.FileSize, m.Manufacturer, m.Version, MD5OfFile(s))
  i = i + 1
 next
next
MainModules = true
end function

sub Main
'msgbox MD5OfFile("G:\Program Files\JediAcademy\GameData\base\assets2.pk3")
'exit sub
IsModules = false
' there is no short-circuit boolean evalution in VB Scripts
if not MainModules then if not MainFiles then exit sub
OutFile.Close
dim r
r = MsgBox(Format(sFinished, OutFilePath), vbYesNo or vbExclamation, sScriptName)
if r <> vbYes then exit sub
WScript.CreateObject("WScript.Shell").Run("""" & OutFilePath & """")
end sub

Main
